import simtk
import simtk.unit
import simtk.openmm
import simtk.openmm.app
import itertools

box_size=10
for lim,simulation_platform,force_type in itertools.product(["0.9999","0.99999999","1"],["CPU","OpenCL","Reference"],["CustomNonbondedForce2","CustomNonbondedForce","CustomHbondForce"]):
    # Create the system
    import simtk.openmm
    pdb=simtk.openmm.app.PDBFile(f'test_initial.pdb')
    top=pdb.topology
    coord=pdb.positions
    forcefield=simtk.openmm.app.ForceField('Particles.xml')
    s=forcefield.createSystem(top)
    s.setDefaultPeriodicBoxVectors([box_size,0,0],[0,box_size,0],[0,0,box_size])

    #Define forces


    if force_type=="CustomNonbondedForce":
        nonbondedForce=simtk.openmm.CustomNonbondedForce(f"""acos(cost3lim);
                                                            cost3lim = min(max(cost3,-{lim}),{lim});
                                                            cost3=4*(r-1)^2-2;""")
    elif force_type=="CustomNonbondedForce2":
        nonbondedForce=simtk.openmm.CustomNonbondedForce(f"acos({lim})")
    else:
        nonbondedForce=simtk.openmm.CustomHbondForce(f"""acos(cost3lim);
                                                        cost3lim = min(max(cost3,-{lim}),{lim});
                                                        cost3=4*(r-1)^2-2;
                                                        r=distance(a1,d1)""")


    nonbondedForce.setCutoffDistance(2.1)


    for a in range(len(pdb.positions)):
        if force_type!="CustomHbondForce":
            nonbondedForce.addParticle()
        else:
            if a % 2==0:
                nonbondedForce.addDonor(a,-1,-1,[])
            else:
                nonbondedForce.addAcceptor(a,-1,-1,[])

    #Add forces
    s.addForce(nonbondedForce)

    import sys
    temperature=300 * simtk.openmm.unit.kelvin

    integrator = simtk.openmm.LangevinIntegrator(temperature, 10 / simtk.openmm.unit.picosecond, 2 * simtk.openmm.unit.femtoseconds)
    platform = simtk.openmm.Platform.getPlatformByName(simulation_platform)
    simulation = simtk.openmm.app.Simulation(top,s, integrator, platform)
    simulation.context.setPositions(coord)
    energy_unit=simtk.openmm.unit.kilojoule_per_mole
    state = simulation.context.getState(getEnergy=True)
    energy = state.getPotentialEnergy().value_in_unit(energy_unit)
    #print(energy)
    simulation.reporters.append(simtk.openmm.app.DCDReporter(f'test.dcd', 10),)
    #simulation.reporters.append(simtk.openmm.app.StateDataReporter(sys.stdout, 10, step=True,time=True,potentialEnergy=True, temperature=True,separator='\t',))
    simulation.reporters.append(simtk.openmm.app.StateDataReporter(f'test.log', 10, step=True,time=True,totalEnergy=True, kineticEnergy=True, potentialEnergy=True, 
                                                                   speed=True, elapsedTime=True, remainingTime=True, temperature=True,totalSteps=1000))
    print(f"\nRunning: Simulation in platform {simulation_platform} using force {force_type} with limit {lim}")
    try:
        simulation.minimizeEnergy()
        simulation.context.setVelocitiesToTemperature(temperature)
        simulation.step(1000)
        print(f"(Completed)")
    except Exception as e:
        print(f"=====Failed=====\n{e}")
